﻿using System;
using System.Collections.Generic;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;

namespace TextAnalyticsDemo
{
    internal struct Document
    {
        public string Id;
        public string Text;
    }

    internal class Program
    {
        private const string AccessKey = "286a83fc3b2d492ab0b677ef27224906";
        private const string UriBase = "https://westus.api.cognitive.microsoft.com/text/analytics/v2.0/";

        private static void Main()
        {
            GetLanguages();
            GetSentiments();
            GetKeyPhrases();

            Console.WriteLine("\nPlease wait a moment for the results to appear. Then, press Enter to exit...\n");
            Console.ReadLine();
        }

        private static async void GetLanguages()
        {
            var documents = new List<Document>
            {
                new Document {Id = "1", Text = "This is a document written in English."},
                new Document {Id = "2", Text = "Este es un document escrito en Español."},
                new Document {Id = "3", Text = "这是一个用中文写的文件"}
            };

            var client = new HttpClient();

            // Request headers
            client.DefaultRequestHeaders.Add("Ocp-Apim-Subscription-Key", AccessKey);

            HttpResponseMessage response;

            // Compose request.
            var body = "";
            foreach (var doc in documents)
            {
                if (!string.IsNullOrEmpty(body))
                {
                    body = body + ",";
                }

                body = body + "{ \"id\":\"" + doc.Id + "\",  \"text\": \"" + doc.Text + "\"   }";
            }

            body = "{  \"documents\": [" + body + "] }";

            // Request body
            var byteData = Encoding.UTF8.GetBytes(body);

            using (var content = new ByteArrayContent(byteData))
            {
                content.Headers.ContentType = new MediaTypeHeaderValue("application/json");
                response = await client.PostAsync(UriBase + "languages", content);
            }

            // Get the JSON response
            var result = await response.Content.ReadAsStringAsync();

            Console.OutputEncoding = Encoding.UTF8;
            Console.WriteLine("\nResponse:\n");
            Console.WriteLine(JsonPrettyPrint(result));
        }

        private static async void GetSentiments()
        {
            var documents = new List<Document>
            {
                new Document {Id = "1", Text = "I really enjoy going to the .NET SIG. It always has good information, and the speakers are well prepared and articulate."},
                new Document {Id = "2", Text = "I enjoy going to the .NET SIG. It has good information, and the speakers are prepared and articulate."},
                new Document {Id = "3", Text = "I enjoy going to the .NET SIG. It usually has good information, but the speakers are sometimes unprepared and occasionally stutter."},
                new Document {Id = "4", Text = "I enjoy going to the .NET SIG. Yrghhh objuuty flbdsgt oonlegij."},
                new Document {Id = "5", Text = "I hate going to the .NET SIG. What a waste of time!"},
                new Document {Id = "6", Text = "I enjoy going to the .NET SIG."},
                new Document {Id = "7", Text = "It usually has good information, but the speakers are sometimes unprepared and occasionally stutter."},

            };

            var client = new HttpClient();

            // Request headers
            client.DefaultRequestHeaders.Add("Ocp-Apim-Subscription-Key", AccessKey);

            HttpResponseMessage response;

            // Compose request.
            var body = "";
            foreach (var doc in documents)
            {
                if (!string.IsNullOrEmpty(body))
                {
                    body = body + ",";
                }

                body = body + "{ \"language\": \"" + "en" + "\", \"id\":\"" + doc.Id + "\",  \"text\": \"" + doc.Text + "\"   }";
            }

            body = "{  \"documents\": [" + body + "] }";

            // Request body
            var byteData = Encoding.UTF8.GetBytes(body);

            using (var content = new ByteArrayContent(byteData))
            {
                content.Headers.ContentType = new MediaTypeHeaderValue("application/json");
                response = await client.PostAsync(UriBase + "sentiment", content);
            }

            // Get the JSON response
            var result = await response.Content.ReadAsStringAsync();

            Console.OutputEncoding = Encoding.UTF8;
            Console.WriteLine("\nResponse:\n");
            Console.WriteLine(JsonPrettyPrint(result));
        }

        private static async void GetKeyPhrases()
        {
            var documents = new List<Document>
            {
                new Document {Id = "1", Text = "I really enjoy going to the .NET SIG. It always has good information, and the speakers are well prepared and articulate."},
                new Document {Id = "2", Text = "I enjoy going to the .NET SIG. It has good information, and the speakers are prepared and articulate."},
                new Document {Id = "3", Text = "I enjoy going to the .NET SIG. It usually has good information, but the speakers are sometimes unprepared and occasionally stutter."},
                new Document {Id = "4", Text = "I enjoy going to the .NET SIG. Yrghhh objuuty flbdsgt oonlegij."},
                new Document {Id = "5", Text = "I hate going to the .NET SIG. What a waste of time!"},
            };

            var client = new HttpClient();

            // Request headers
            client.DefaultRequestHeaders.Add("Ocp-Apim-Subscription-Key", AccessKey);

            HttpResponseMessage response;

            // Compose request.
            var body = "";
            foreach (var doc in documents)
            {
                if (!string.IsNullOrEmpty(body))
                {
                    body = body + ",";
                }

                body = body + "{ \"language\": \"" + "en" + "\", \"id\":\"" + doc.Id + "\",  \"text\": \"" + doc.Text + "\"   }";
            }

            body = "{  \"documents\": [" + body + "] }";

            // Request body
            var byteData = Encoding.UTF8.GetBytes(body);

            using (var content = new ByteArrayContent(byteData))
            {
                content.Headers.ContentType = new MediaTypeHeaderValue("application/json");
                response = await client.PostAsync(UriBase + "keyPhrases", content);
            }

            // Get the JSON response
            var result = await response.Content.ReadAsStringAsync();

            Console.WriteLine("\nResponse:\n");
            Console.WriteLine(JsonPrettyPrint(result));
        }

        private static string JsonPrettyPrint(string json)
        {
            if (string.IsNullOrEmpty(json))
                return string.Empty;

            json = json.Replace(Environment.NewLine, "").Replace("\t", "");

            var sb = new StringBuilder();
            var quote = false;
            var ignore = false;
            var offset = 0;
            const int indentLength = 3;

            foreach (var ch in json)
            {
                switch (ch)
                {
                    case '"':
                        if (!ignore) quote = !quote;
                        break;
                    case '\'':
                        if (quote) ignore = !ignore;
                        break;
                }

                if (quote)
                    sb.Append(ch);
                else
                {
                    switch (ch)
                    {
                        case '{':
                        case '[':
                            sb.Append(ch);
                            sb.Append(Environment.NewLine);
                            sb.Append(new string(' ', ++offset * indentLength));
                            break;
                        case '}':
                        case ']':
                            sb.Append(Environment.NewLine);
                            sb.Append(new string(' ', --offset * indentLength));
                            sb.Append(ch);
                            break;
                        case ',':
                            sb.Append(ch);
                            sb.Append(Environment.NewLine);
                            sb.Append(new string(' ', offset * indentLength));
                            break;
                        case ':':
                            sb.Append(ch);
                            sb.Append(' ');
                            break;
                        default:
                            if (ch != ' ') sb.Append(ch);
                            break;
                    }
                }
            }

            return sb.ToString().Trim();
        }

    }
}
